@@ -234,13 +234,14 @@ def do_forward_request(api, method, path, data, headers, port=None):
     return result
 
 
+def get_handler_for_api(api, headers):
+    return PROXY_LISTENERS.get(api)
+
+
 def do_forward_request_inmem(api, method, path, data, headers, port=None):
-    listener_details = PROXY_LISTENERS.get(api)
+    listener_details = get_handler_for_api(api, headers)
     if not listener_details:
-        message = (
-            'Unable to find listener for service "%s" - please make sure to include it in $SERVICES'
-            % api
-        )
+        message = f'Unable to find listener for service "{api}" - please make sure to include it in $SERVICES'
         LOG.warning(message)
         raise HTTPErrorResponse(message, code=400)
     service_name, backend_port, listener = listener_details
@@ -372,6 +373,11 @@ def get_api_from_headers(headers, method=None, path=None, data=None):
         result = "resourcegroupstaggingapi", config.PORT_RESOURCEGROUPSTAGGINGAPI
     elif result[0] == "resource-groups":
         result = "resource-groups", config.PORT_RESOURCE_GROUPS
+    elif result[0] == "es" and path is not None and not path.startswith("/2015-01-01/"):
+        # For OpenSearch, the auth header points to the API ("es").
+        # However, if the path does _not_ start with /2015-01-01 (the API version path prefix for the only ES API
+        # version) it is a request to the opensearch API.
+        result = "opensearch", config.PORT_OPENSEARCH
 
     return result[0], result_before[1] or result[1], path, host
 
@@ -453,15 +459,12 @@ def get_api_from_custom_rules(method, path, data, headers):
 
     # TODO: move S3 public URLs to a separate port/endpoint, OR check ACLs here first
     stripped = path.strip("/")
-    if method in ["GET", "HEAD"] and "/" in stripped:
+    if method in ["GET", "HEAD"] and stripped:
         # assume that this is an S3 GET request with URL path `/<bucket>/<key ...>`
         return "s3", config.PORT_S3
 
     # detect S3 URLs
     if stripped and "/" not in stripped:
-        if method == "HEAD":
-            # assume that this is an S3 HEAD bucket request with URL path `/<bucket>`
-            return "s3", config.PORT_S3
         if method == "PUT":
             # assume that this is an S3 PUT bucket request with URL path `/<bucket>`
             return "s3", config.PORT_S3
